#!/usr/bin/wish

catch {package require Tclx}
set noFvwm [catch {package require Fvwm}]
set CONFIGDIR "~/.fubar"
set LIBRARYDIR [file dirname [info script]]
# Host add setup
set accessCommand [list rs ssh rsh]
set LocaleList {ru_RU.KOI8-R ru_RU.CP1251 ru_RU.UTF-8}

option add *Menu.Font -*-times-bold-r-normal--14-* widgetDefault
option add *Menubutton.Font -*-times-bold-r-normal--14-* widgetDefault
set WindowMenu "Window-Ops2"
source [file join $LIBRARYDIR balloonhelp.tcl]
option add Foobar.geometry +0+0 widgetDefault
# No periodically executed commands by default
set scheduled_commands {}
#
# Lays out an application
#
proc myExit {args} {exit}
proc main {} {
   # Just for convinience
   global CONFIGDIR LIBRARYDIR
   # First, transform main window into bar
   setup_window
   # Second, check if our config dir exist. Otherwise 
   if {![file exist $CONFIGDIR] } {
      uplevel #0 source [file join $LIBRARYDIR setup_userdir]
   } elseif {![file isdirectory $CONFIGDIR]} {
      tk_messagebox -type ok -title "Error" -icon error -message "Config directory $CONFIGDIR missing and cannot be created!"
      exit 1
   }    
   read_hostfile
   read_associations
   make_startmenu
   make_windowmenu
   make_runbutton
   make_termmenu
   make_findmenu
   read_hotkeys	
  	after idle load_plugins
   # Start tasks to be executed each minute
   bind . <Button-1> {raise .}
   scheduler
}

proc load_plugins {} {
   global CONFIGDIR
   foreach plugin [glob -nocomplain $CONFIGDIR/plugins/*] {
	  if {[string match *.txt $plugin]} continue
      if {[catch {uplevel #0 [list source $plugin]} msg]} {
	   	  bgerror  "Error loading plugin [file tail $plugin]\n$msg"
	  }	  
   }
} 
#
# Sets main window up as horizontal bar
#
proc setup_window {} {
  . configure -relief raised -bd 3
  if {$::noFvwm} {
     wm overrideredirect . yes
  } else {
  	  ::fvwm::send "Style [tk appname] NoTitle,NoHandles,BorderWidth 0,WindowListSkip, Sticky "
  }	  
  regexp {[0-9+]x[0-9+]([+-][0-9]+)([+-][0-9]+)} [wm geometry .] junk xpos ypos
  wm geometry . [winfo screenwidth .]x30+0$ypos
  update
} 
##############################################################
# Reading universal configuration
#############################################################
#
# Reads file of known hosts used by run window and term menu
#

proc read_hostfile {} {
	global HostList CONFIGDIR RunCmd XtermCmd Xterm hostTimeStamp
		if [catch {open $CONFIGDIR/hosts} f] {
#file not found, create default one
			tk_messageBox -title Warning -type ok -message "Cannot open host list: $f\nCreating default one"
			set f [open $CONFIGDIR/hosts w]
			puts $f "#hostname rsh-command locale list"
			puts $f "localhost - $::env(LANG)"
			close $f
			set f [open $CONFIGDIR/hosts]
		}
     set HostList {}
     while {[gets $f line]>=0} {  
        if {![string length $line]||[regexp "^ *#" $line]} continue
		if {[regexp "^\[ \t]*-+" $line]} {
			lappend HostList "-"
		} elseif {[regexp "^\[ \t]*>+\[ \t]*(.*)" $line => label]} {
			lappend HostList ">$label"
		} elseif {[regexp "^\[ \t]*<" $line]} {
			lappend HostList "<"
		} else {	
			set list [regexp -all -inline {[^[:space:]]+} $line]
			set host [lindex $list 0]
			set cmd [lindex $list 1]
			set locales [lrange $list 2 end] 
			if {![llength $locales]} {
				set locales [list $::env(LANG)]
			}
			set connect_name ""
			foreach {menu_name connect_name} [split $host ":"] break
			if {![string length $connect_name]} {
				set connect_name $menu_name
			}
			if {[lindex $locales 0]==">"} {
				set locales [lrange $locales 1 end]
				set locales_submenu 1
				lappend HostList ">$menu_name"
			} else {
				set locales_submenu 0
			}	
			set loccount [llength $locales]
			foreach l $locales {
				if {$l eq $::env(LANG)} {
					set lang_prefix ""
				} else {
					set lang_prefix "env LANG=$l "
				}
				if {$loccount >1} {
					catch {unset charset}
					foreach {lang charset} [split $l "."] break
					if {[info exists charset]} {
						set name "$menu_name\([string tolower $charset]\)"
					} else {
						set name "$menu_name\($lang\)"
					}
				} else {
					set name $menu_name
				}	
				if {$cmd eq "-"} {
					set rexec ""
					set xexec ""
					set xtermswitch " -e "
				} else {
					set rexec "$cmd $connect_name"
					set xexec "-e $cmd $connect_name"
					set xtermswitch ""
				}	
				lappend HostList $name
				set RunCmd($name) [string trim "$lang_prefix$rexec"]
				set Xterm($name) "$lang_prefix xterm -T \"Shell on $name\" $xexec"
				set XtermCmd($name) $Xterm($name)$xtermswitch	
			}	
			if {$locales_submenu} {
				lappend HostList "<"
			}	
	    }
	 }
	 set hostTimeStamp [clock seconds]
}

#
# Checks mtime of hosts file and, if changed, rereads it
#
proc check_hostFile {} {
    global hostTimeStamp CONFIGDIR
    if {[file mtime $CONFIGDIR/hosts]>$hostTimeStamp} {
        read_hostfile
		build_xtermmenu
		build_runhostmenu
    }
}   

proc read_hotkeys {} {

}

proc build_runhostmenu {} {
   global runHost HostList 
   build_host_menu .runwindow.host.menu radiobutton {-value $host -variable runHost}
}   
#
# Read file of associations
#
proc read_associations {} {
   global Associations CONFIGDIR noFvwm
   array set Associations {
       .tcl builtin
   }
   if {!$noFvwm} {
      set Associations(.fvwm) builtin
   }   
   if [file readable $CONFIGDIR/associations] {
      set f [open $CONFIGDIR/associations]
      catch {array set Associations [read -nonewline $f]}
      close $f     
   }
}
######################################################################
# Start menu stuff
######################################################################

#
# Creates start menu (which is actually recreated each time as posted)
#
proc make_startmenu {} {
    global CONFIGDIR
    menubutton .start -menu .start.menu -text Start -relief raised -bd 2 
    menu .start.menu -tearoff false -postcommand scan_appdir
    pack .start -side left -ipady 3
    balloonhelp .start "Click here to begin ;-)"
	menu .start.setup -tearoff 0
	.start.setup add command -label "Open" -command [list exec\
		$CONFIGDIR/fm [file normalize $CONFIGDIR/menu] &]
	if {![file executable $CONFIGDIR/fm]} {
		.start.setup entryconfigure "Open" -state disabled
	}	
	.start.setup add command -label "Reread hotkeys..." -command "read_hotkeys"
	.start.setup add command -label "Import history..." -command "import_history"
     bind .start <3> [list tk_popup .start.setup %X %Y]
}
#
# Deletes all items from menu and all associated submenus
#
proc deleteCascade {menu} {
    while {[$menu index end]!="none"} {
        if {[$menu type 0]=="cascade"} {
            set submenu [$menu entrycget 0 -menu]
            deleteCascade $submenu
            destroy $submenu
        }
        $menu delete 0
    }
}

#
# Scans given directory and forms menu
#
proc make_cascade {path menu} {
   global XtermCmd Associations HostList
  	set runHost [lindex $HostList 0]
   set counter 0
   foreach f [lsort -dictionary [glob -nocomplain $path/*]] {
       set name [file tail $f]
       if [file isdirectory $f] {
          set submenu $menu.$counter
          incr counter
          $menu add cascade -label [mklabel $name] -menu $submenu
          menu $submenu -tearoff n
          make_cascade $f $submenu
       } else {    
	   set state normal
	   if [file executable $f] {
	       if {[file extension $f]==".xterm"} {
		  set name [file rootname $name]
		  set command "exec $XtermCmd($runHost) [list $f] >&@ stdout &"
	       } else {
		  set command "exec [list $f] >&@ stdout &"
	       }
	   } else {
	      set command "launch [list $f]"
	      if ![info exist Associations([file extension $f])] {
		 set state disabled
	      } else {
		 set name [file rootname $name]
	      }
	  }
          $menu add command -label [mklabel $name] -command $command -state $state
	}
   }
}    
#
# Converts file name to menu label - strips off leading number if any
# and casts underscores to spaces
proc mklabel {name} {
  if [regexp {^[0-9]*\.(.*)$} $name tt label] {
     set name $label
  }
  regsub {_} $name " " label
  return $label
}
#
# import_history - creates Start menu items from run menu history
#
proc import_history {} {
	tk_messageBox -type ok -title "Import history" -message "Not implemented yet"
}


#
# launch - invokes non-executable file which have associations
#
proc launch {file} {
   global Associations 
   set ext [file extension $file]
   switch -exact $ext {
     .tcl {
          uplevel #0 source [list $file]
     }
     .fvwm {
           set f [open $file]
           set list [split [read -nonewline $f] "\n"]
           close $f
           foreach command $list {
               if [regexp "\[ \t]*#" $command] continue 
               ::fvwm::send $command
           }
      }
      default {
            exec /bin/sh -c [format $Associations($ext) $file] >&@stdout &
      }
   }
} 

proc scan_appdir {} {
   global CONFIGDIR
   deleteCascade .start.menu
   make_cascade  $CONFIGDIR/menu .start.menu
}
######################################################################
# Window list hangling - only if invoked as Fvwm Module
######################################################################
#
# Creates window menu which is actually recreated each time as posted
#
proc make_windowmenu {} {
    global noFvwm
    menubutton .windows -menu .windows.menu -text Windows -relief raised -bd 2
    menu .windows.menu -tearoff false -postcommand getwinlist
    bind .windows.menu <3> {winop_menu [.windows.menu index @%y]}
    pack .windows -side left
    if $noFvwm {
          .windows configure -state disabled
    }
    balloonhelp .windows "Click here to list open windows"
    balloonhelp .windows.menu "Left - raise right- winops menu"
}
#
# Rescans current windows and forms window menu
#
proc getwinlist {} {
    global WindowIds
    while {[.windows.menu index 0] != "none"} {
       .windows.menu delete 0
    }   
    catch {unset WindowIds}
    ::fvwm::getWindowList list
    foreach idx [array names list *,iconName] {
       set id [lindex [split $idx ","] 0 ]
       if [info exist list($id,iconic)] {
         set label "<$list($idx)>"
	 if {[string first "\0" $label"]>=0} {
	    regsub -all "\0" $label {} label
	    puts stderr "Problem with window title \"$label\""
	 }   
	 set cmd "Iconify -1"
       } else {
         set label $list($idx)
		 if {$label == "[tk appname]"} continue
	 regsub -all "\0" $label {} label
	 set cmd "MyFocus"
       }	 
       .windows.menu add command -label $label -command \
	      [list ::fvwm::send $cmd $id]
       #balloonhelp .windows.menu -index [.windows.menu index end] $list($id,title) 	      
       set WindowIds([.windows.menu index end]) $id   	  
	      
   } 
}
#
# Executes executes fvwm window-ops menu on given command
#
proc winop_menu {index} {
    if {$index=="none"} return
    global WindowIds WindowMenu
    ::fvwm::send "Popup $WindowMenu" $WindowIds($index)
}    
#
# Button which opens a popup window to enter command. Note that window
# is created and only shown from this menu. See section about POPUP
# WINDOWS
#
proc make_runbutton {} {
    button .run -text Run -relief raised -pady 1 \
      -command {show_popup .runwindow .run; focus -force .runwindow.command}
    pack .run -side left
    balloonhelp .run "Click here for single command prompt"
    make_run_window
    
}
#
# Creates run window
#
proc make_run_window {} {
  global runHost HostList runInXterm pauseAfterRun
  set runHost [lindex $HostList 0]
  set runInXterm 0
  set pauseAfterRun 0
  popup .runwindow -relief raised -bd 2
# Things to have inside run window  
  label .runwindow.l -text "Run:"
  entry .runwindow.command -width 50 -textvar runCommand
  bind .runwindow.command <Return> runACommand
  menubutton .runwindow.history -text "v" -relief raised -bd 2 \
	-menu .runwindow.history.menu
  balloonhelp .runwindow.history "Click to choose from list of previous commands"
  menu .runwindow.history.menu -tearoff no
  read_history .runwindow.history.menu
  button .runwindow.browse -text "Browse..." -command browse_file
  balloonhelp .runwindow.browse "Find a file to substitute in command line"
  label .runwindow.l2 -text "Run on:"
  global runHost
  menubutton .runwindow.host -textvar runHost -relief raised -indicatoron y -menu .runwindow.host.menu
  menu .runwindow.host.menu 
  build_runhostmenu
  # eval tk_optionMenu .runwindow.host runHost $HostList
  trace var runHost w checkHostIsLocal
  .runwindow.host.menu configure -postcommand check_hostFile
  balloonhelp .runwindow.host "Select host to run on"
  checkbutton .runwindow.xterm -text "Run in xterm" -command \
       pauseButtonState -variable runInXterm -anchor w
  checkbutton .runwindow.pause -text "Don't close xterm on exit"\
	-variable pauseAfterRun -state disabled -anchor w       
  button .runwindow.run -command runACommand -text "Run"
  balloonhelp .runwindow.run "Click here to execute"
  button .runwindow.cancel -command {hide_popup .runwindow} -text Cancel
  balloonhelp .runwindow.run "Click to forget about this command"
  grid .runwindow.l -padx 10 -sticky wns
  grid .runwindow.command - - - .runwindow.history -sticky news -padx 5
  grid .runwindow.l2 .runwindow.host -sticky news -padx 10
  grid x .runwindow.xterm - -sticky news
  grid x .runwindow.pause - -sticky news
  grid x  .runwindow.browse .runwindow.run .runwindow.cancel -padx 10 -sticky news 
  focus .runwindow.command
}

proc checkHostIsLocal {args} {
   global runHost
   if {[string match localhost* $runHost]} {
       .runwindow.browse configure -state normal
   } else {
       .runwindow.browse configure -state disabled
   }
}   
#
# Reads history file (invoked once on startup)
#

proc read_history {menu} {
  global CONFIGDIR
  if [catch {open $CONFIGDIR/history} f] return
  while {[gets $f line]>=0} {
    if {[regexp {^([^ ]+) +([0-9]+) +([0-9]+) +(.*)$} \
             $line junk host xt pause cmd]} {
       $menu add command -label $cmd -command \
               [list set_command $cmd $host $xt $pause] }
  }
  close $f
}
#
# Sets widgets in run window to command got from history
#
proc set_command {cmd host xt pause} {
  global runCommand  runInXterm pauseAfterRun runHost
  set runHost $host
  set runCommand $cmd
  set runInXterm $xt
  set pauseAfterRun $pause
}
#
# Adds current command into history and saves history to file
#
proc add_history {} {
  global CONFIGDIR runHost runCommand  runInXterm pauseAfterRun
  catch {.runwindow.history.menu delete $runCommand}
  .runwindow.history.menu insert 0 command -label $runCommand \
       -command [list set_command $runCommand $runHost $runInXterm\
            $pauseAfterRun]
  while {[.runwindow.history.menu index end]>20} {
     .runwindow.history.menu delete end
  }   
  set f [open $CONFIGDIR/history w]
  for {set i 0} "\$i<=[.runwindow.history.menu index end]" {incr i} {
     set list [.runwindow.history.menu entrycget $i -command]
     puts $f "[lindex $list 2] [lindex $list 3]  [lindex $list 4] [lindex $list 1]"
  }
  close $f;
}
#
# Pops up file dialog and inserts choosen name in place of selection
# into command line
#
proc browse_file {} {
  set name [tk_getOpenFile -parent .runwindow -title Browse]
  set e .runwindow.command
  if {"$name"==""} return
  if [$e selection present] {
     $e delete sel.first sel.last
     $e insert selfirst $name
  } else {   
     $e insert insert $name
  }
}  
#
# Executes command entered in runwindow
#
proc runACommand {} {
    global RunCmd XtermCmd runCommand runHost runInXterm pauseAfterRun
    hide_popup .runwindow
    add_history

    if $runInXterm {
       if $pauseAfterRun {
         set command "/bin/sh -c '$runCommand;echo \"Hit <Return>\";read junk'" 
       } else {
         set command $runCommand
       }
        exec /bin/sh -c "$XtermCmd($runHost) $command" &>@stdout & 
    } else {
       if {$RunCmd($runHost)=="*"} {
          tk_messageBox -title Error -type Ok \
	       -message "You cannot run X applications on $runHost"
          return
       }	       
       exec /bin/sh -c "$RunCmd($runHost) $runCommand" &>@stdout &
    }   
    set runCommand {}
}
#
# Change state of pause button
#
proc pauseButtonState {} {
  global runInXterm 
  if $runInXterm {
     set state normal
  } else {
     set state disabled
  }
  .runwindow.pause configure -state $state
}  
##############################################################
# Popup borderless windows
##############################################################
proc popup {widget args}  {
   array set widget_args $args
   set need_bind 0
   if {![info exists widget_args(-class)]} {
    	set widget_args(-class) Popup
   } else {
    	set need_bind 1
   }		
   set w [eval toplevel $widget [array get widget_args ]]
   if {$need_bind} {
   	 	foreach event [bind Popup] {
			bind $widget_args(-class) $event [bind Popup $event]
		}
   }	
		
   wm overrideredirect $w yes
   wm withdraw $w 
}

proc show_popup {window origin} {
   set x  [winfo rootx $origin]
   set y  [expr [winfo rooty $origin]+[winfo height $origin]]
   if {$y+[winfo reqheight $window]>[winfo screenheight $window]} {
     set y [expr [winfo rooty $origin] - [winfo reqheight $window]]
     if {$y<0} {
        set y [expr [winfo screenheight $window]-[winfo reqheight $window]] 
	if {$y<0} {
	    set y 0
	}
     }
   }
   if {$x+[winfo reqwidth $window]>[winfo screenwidth $window]} {
     set x [expr [winfo screenwidth $window]-[winfo reqwidth $window]]
     if {$x<0} {
       set x 0
     }
   }  
   wm geometry $window +$x+$y
   wm deiconify $window
   raise $window
   grab -global $window
}   

proc popupLeft {window x y} {
  if {$x<0||$y<0||$x>[winfo width $window]||$y>[winfo height $window]} {
     hide_popup $window
  }
}

proc hide_popup window {
  grab release $window
  wm withdraw $window
}  
bind Popup <Button-1> {popupLeft %W %x %y}
bind Popup <Button-2> {popupLeft %W %x %y}
bind Popup <Button-3> {popupLeft %W %x %y}
##############################################################
# Menu of xterms on all possible hosts
##############################################################

#
# Creates menu for invoke shell window on any of known hosts
#
proc make_termmenu {} {
  menubutton .xterm -text "Xterm" -menu .xterm.menu -bd 2 -relief raised
  bind .xterm <ButtonRelease-3> {create_or_raise .hostlist edit_hostlist}
  menu .xterm.menu -tearoff n -postcommand check_hostFile
  build_xtermmenu
  balloonhelp .xterm "Click to open new terminal window"
}  
proc build_xtermmenu {} {
  global Xterm
  build_host_menu .xterm.menu command {-command {exec $Xterm($host) >&@stdout &}}
  pack .xterm -side left
}

proc build_host_menu {menu type options } {
  global HostList Xterm
  set top_menu $menu
  $menu delete 0 end
  catch [concat destroy [winfo children $menu]]
  set menu_stack {}
  set submenu 0
  foreach host $HostList {
      if { $host == "-" } {
        $menu add separator
	  } elseif {[regexp "^>(.*)$" $host => label]} {
		lappend menu_stack $menu
		$menu add cascade -label $label -menu [set menu [menu $top_menu.m[incr submenu]]]
      } elseif { $host == "<" } {
	  	set menu [lindex $menu_stack end]
		set menu_stack [lrange $menu_stack 0 end-1] 
      } else {	
        eval [list $menu add $type -label $host ] [subst " $options"]
	  }
  }
}

#
# start periodic tasks like menu update
#
proc scheduler {} {
    global scheduled_commands
    foreach cmd $scheduled_commands {
	uplevel #0 $cmd [clock seconds]
    }    
    set sec [string trimleft [clock format [clock seconds] -format "%S"] 0]
    if ![string length $sec] {
       set sec 0
    }   
    after [expr (60-$sec)*1000] scheduler
}
#
# Allows plugin to register proc which would be run each minute
#
proc notifier {command} {
   global scheduled_commands
   lappend scheduled_commands $command
}   
#
# Find menu - allows to find out lot of useful things about system
#

proc make_findmenu {} {
menubutton .find -text Find -relief raised -bd 2 -menu .find.m
balloonhelp .find "Find a useful information about..."
set m [menu .find.m -tearoff no]
$m add command -label "File..." -command "create_or_raise\
	.find_file find_file; focus .find_file.locate"
#balloonhelp $m -index 0 "Search for a file on the filesystem"
$m add command -label "Host..." -command "create_or_raise\
	.find_host find_host;focus .find_host.addr"
#balloonhelp $m -index 3 "Search for a person in the phonebook or passwd file"
$m add command -label "Command..." -command "create_or_raise\
        .find_command find_command; focus .find_command.cmd"
#balloonhelp $m -index 4 "Search for command man page (apropos)"	
if {![findInPath "apropos"]} {
    $m entryconfigure "Command..." -state disabled
}    
pack .find -side left
}

proc findInPath {cmd} {
    global env
    foreach dir [split $env(PATH) ":"] {
       if [file executable [file join $dir $cmd]] {
             return 1
       }
    }
    return 0
}
proc create_or_raise {w fillproc args} {
  if [winfo exists $w] {
    if {[wm state $w] != "normal"} {
      wm deiconify $w
    } else {
      raise $w
      focus $w
    }
    return 0
  } else {  
    eval toplevel $w $args
    $fillproc $w
  }
  return 1
}

proc find_file {w} {
     wm title $w "Find file"
     label $w.l1 -text "Locate:" -anchor w
     entry $w.locate -width 30
     bind $w.locate <Key-Return> [list do_locate $w]
     button $w.do_locate -text "Locate" -command [list do_locate $w]
     grid $w.l1 $w.locate $w.do_locate - -sticky news
     label $w.l2 -text "Find:" -anchor w
     grid $w.l2 x x x
     entry $w.find -width 50 
     bind $w.find <Key-Return> [list do_find $w]
     button $w.do_find -text "Find" -command [list do_find $w]
     grid $w.find - $w.do_find - -sticky news
     text $w.result -height 5 -yscrollcommand [list $w.y set] -state disabled
     $w.result tag configure error -foreground red
     $w.result tag configure done -foreground green 
     scrollbar $w.y -orient vert -command [list $w.result yview]
     grid $w.result - - $w.y -sticky news
     grid columnconfigure $w 0 -weight 0
     grid columnconfigure $w 1 -weight 1
     frame $w.b
     grid $w.b - - - - -sticky ns
     button $w.b.stop -text Stop -state disabled
     button $w.b.clear -text Clear -command [list clearresult $w.result]
     button $w.b.close -text Close -command [list find_close $w $w.b.stop]
     pack $w.b.stop $w.b.clear $w.b.close -side left
     wm protocol $w WM_DELETE_WINDOW [list find_close $w $w.b.stop]
     foreach row {0 1 2 4} {
         grid rowconfigure $w $row -weight 0
     }
     grid rowconfigure $w 3 -weight 1
}

proc find_close {window stop} {
   if {[$stop cget -state] == "normal"} {
      $stop invoke
   }
   destroy $window
}

proc read_cmd {channel window stopbutton} {
   if [eof $channel] {
       $window config -state normal
       if [catch {close $channel} error] {
	  $window insert end $error error
       } else {
          $window insert end "****** done ******" done
       }	  
       $window config -state disabled
       if [string length $stopbutton] {
           $stopbutton configure -state disabled
       }
       return
   }
   $window config -state normal
   $window insert end [read $channel]
   $window see end
   $window config -state disabled
}

proc open_pipe {window command {stopbutton {}}} {
    $window configure -state normal
    $window delete 0.0 end
    $window configure -state disabled
    set f [open "|$command" r]
    fconfigure $f -blocking no -buffering line
    if [string length $stopbutton] {
       $stopbutton configure -state normal -command [list kill [pid $f]]
    }   
    fileevent $f readable [list read_cmd $f $window $stopbutton]
}
     
proc do_locate {window} {
   set pattern [$window.locate get]
   open_pipe $window.result "locate $pattern" $window.b.stop
}

proc do_find {window} {
   set expression [$window.find get]
   open_pipe $window.result "find $expression" $window.b.stop
}   

proc find_host {w} {
  wm title $w "Find host"
  label $w.l -text "Enter hostname or IP address:" -anchor w
  entry $w.addr -width 30 
  bind $w.addr <Key-Return> [list do_dns_lookup $w]
  label $w.l2 -text "Record type:"
  button $w.lookup -text "Lookup DNS" -command [list do_dns_lookup $w]
  button $w.route -text "Traceroute" -command "open_pipe $w.result\
          \"/usr/bin/traceroute \[$w.addr get\]\" $w.b.stop"
  tk_optionMenu $w.type nslookup_type "A/PTR" "MX" "SOA" "NS"
  grid $w.l - - - - -sticky w
  grid $w.addr - - - - -sticky w
  grid $w.l2 $w.type $w.lookup $w.route - -sticky ns
  text $w.result -height 5 -yscrollcommand [list $w.y set] -state disabled
     $w.result tag configure error -foreground red
     $w.result tag configure done -foreground green 
     scrollbar $w.y -orient vert -command [list $w.result yview]
     grid $w.result - - - $w.y -sticky news
     grid columnconfigure $w 0 -weight 0
     grid columnconfigure $w 1 -weight 0
     grid columnconfigure $w 2 -weight 1
     grid columnconfigure $w 3 -weight 0
     grid columnconfigure $w 4 -weight 0
     frame $w.b
     grid $w.b - - - - -sticky ns
     button $w.b.stop -text Stop -state disabled
     button $w.b.clear -text Clear -command [list clearresult $w.result]
     button $w.b.close -text Close -command [list find_close $w $w.b.stop]
     pack $w.b.stop $w.b.clear $w.b.close -side left
     wm protocol $w WM_DELETE_WINDOW [list find_close $w $w.b.stop]
     foreach row {0 1 2 4} {
         grid rowconfigure $w $row -weight 0
     }
     grid rowconfigure $w 3 -weight 1
}

proc do_dns_lookup {w} {
    global nslookup_type;
    if {"$nslookup_type" == "A/PTR"} {
        set command "host"
    } else {
        set command "host -t $nslookup_type"
    }
    open_pipe $w.result "$command [$w.addr get]" $w.b.stop
}    

proc clearresult {w} {
   $w configure -state normal
   $w delete 0.0 end
   $w configure -state disabled
}   


proc find_command {w} {
  wm title $w "Find command in manual"
  label $w.l -text "Command:"
  entry $w.cmd -width 30
  bind $w.cmd <Key-Return> [list $w.find invoke]
  button $w.find -text "Find" -command "do_apropos $w.result \[$w.cmd get\]"
  grid $w.l $w.cmd $w.find - -sticky news
  text $w.result -height 5 -yscrollcommand [list $w.y set] -state disabled\
     -font 6x13
     $w.result tag configure error -foreground red
     $w.result tag configure done -foreground green 
     $w.result tag configure man -foreground blue -underline y
     $w.result tag bind man <1> "show_man $w.result  @%x,%y"
     scrollbar $w.y -orient vert -command [list $w.result yview]
     grid $w.result - -  $w.y -sticky news
     grid columnconfigure $w 0 -weight 0 
     grid columnconfigure $w 1 -weight 1 
     grid columnconfigure $w 2 -weight 0 
     grid columnconfigure $w 3 -weight 0
     foreach row {0 1} {
         grid rowconfigure $w $row -weight 0
     }
     grid rowconfigure $w 2 -weight 1
}
  

proc do_apropos {window command} {
    $window configure -state normal
    $window delete 0.0 end
    $window configure -state disabled
    set f [open "|apropos \"$command\"" r]
    fconfigure $f -blocking no -buffering line
    fileevent $f readable [list read_apropos $f $window]
}

proc read_apropos {channel window} {
   if [eof $channel] {
       $window config -state normal
       if [catch {close $channel} error] {
	  $window insert end $error error
       } else {
          $window insert end "****** nothing more ******" done
       }	  
       $window config -state disabled
       return
   }
   $window config -state normal
   foreach line [split [read $channel] "\n"] {
     if [regexp "^(\[^ \]+ \\(\[^ \]+\\))(.*$)" $line all cmd comment] {
       $window insert end $cmd man "$comment\n" {}
     } else {
       $window insert end $line error "\n" {}
     }  
   }
   $window see end
   $window config -state disabled
}

proc show_man {w index} {
  global CONFIGDIR
  set range [$w tag nextrange man "$index linestart"]
  regexp "^(\[^ \]+) \\((\[^)\]+)\\)" [eval $w get $range] all command section 
  exec $CONFIGDIR/man $section $command &
}


proc edit_hostlist {w} {
  wm title $w "Add host";
  label $w.l1 -text "Menu label" -anchor e
  entry $w.label 
  label $w.l2 -text "Hostname" -anchor e
  entry $w.name 
  label $w.l3 -text "Access command" -anchor e
  eval tk_optionMenu $w.protocol hostCommand $::accessCommand 
  labelframe $w.loc -text "Suppored locales" -labelanchor nw
  set i 0
  foreach locale $::LocaleList {
  	incr i
	checkbutton $w.loc.box$i -text $locale -offvalue "" -onvalue $locale -var hostLocale$i
	pack $w.loc.box$i -side top
  }
  grid $w.l1 $w.label -sticky news
  grid $w.l2 $w.name -sticky news
  grid $w.l3 $w.protocol -sticky news
  grid $w.loc - -sticky news
  button $w.save -text Save -command "add_host_entry $w"
  button $w.cancel -text Cancel -command "wm withdraw $w"
  grid $w.save $w.cancel
}

proc add_host_entry {w } {
global CONFIGDIR hostCommand 
	set hostMenuLabel [$w.label get]
	set hostName [$w.name get]
	if {![string length $hostMenuLabel]} {
		set $hostMenuLabel $hostName
	}
	if {![string length $hostName]} {
		bell
		return
	}
	if {$hostMenuLabel eq $hostName} {
		set line $hostName
	} else {
		set line $hostMenuLabel:$hostName
	}	
	append line " $hostCommand"
	for {set i 1} {[winfo exists $w.loc.box$i]} {incr i} {
		set l [uplevel #0 set hostLocale$i]
		if {[string length $l]} {
			append line " $l"
		}
	}	
	set f [open $CONFIGDIR/hosts a]
	puts $f $line
	close $f
	wm withdraw $w
}
main
